@@ -11,8 +11,7 @@ __all__ = [
 import itertools
 import json
 import logging
-import operator
-from functools import reduce
+from datetime import date, datetime
 
 from pyroonga import utils
 from pyroonga.odm.attributes import SuggestType
@@ -284,6 +283,7 @@ class Drilldown(object):
         self._nsubrecs = _nsubrecs
 
 
+@utils.python_2_unicode_compatible
 class SelectQueryBase(Query, QueryOptionsMixin):
     """'select' query representation base class"""
 
@@ -297,6 +297,7 @@ class SelectQueryBase(Query, QueryOptionsMixin):
         Query.__init__(self, tbl)
         QueryOptionsMixin.__init__(self)
         self._exprs = list(Expression.wrap_expr(*args))
+        self._filters = []
         self._target = kwargs
         self._match_columns = []
         self._cache = True
@@ -317,9 +318,7 @@ class SelectQueryBase(Query, QueryOptionsMixin):
         :param args: iterable of columns
         :returns: self. for method chain
         """
-        from pyroonga.odm.table import Column  # avoid Circular reference
-        columns = (MatchColumn(c) if isinstance(c, Column) else c
-                   for c in args)
+        columns = (Expression(getattr(c, 'name', c)) for c in args)
         self._match_columns.extend(columns)
         return self
 
@@ -334,6 +333,20 @@ class SelectQueryBase(Query, QueryOptionsMixin):
         self._target.update(kwargs)
         return self
 
+    def filter(self, *args, **kwargs):
+        """Set the filter query parameters
+
+        :param args: instances of :class:`ExpressionTree`
+        :param kwargs: Column names and values for Match operator.
+            If more than one given, they will be concatenate with *OR* operator
+        :returns: self. for method chain
+        """
+        filters = self._filters
+        filters.extend(Expression.wrap_expr(*args))
+        filters.extend(Expression(k).match(v) for k, v in
+                       sorted(kwargs.items()))
+        return self
+
     def cache(self, iscache):
         """Set the query cache
 
@@ -354,8 +367,9 @@ class SelectQueryBase(Query, QueryOptionsMixin):
 
     def _make_match_columns(self):
         if self._match_columns:
-            return "--match_columns '%s'" % reduce(operator.or_,
-                                                   self._match_columns)
+            exprs = (e.build(MatchColumn) for e in self._match_columns)
+            result = MatchColumn.operator[Operator.OR].join(exprs)
+            return '--match_columns %s' % utils.escape(result, True)
         else:
             return ''
 
@@ -372,16 +386,24 @@ class SelectQueryBase(Query, QueryOptionsMixin):
     def _makeparams(self):
         return ''
 
+    def _makefilters(self):
+        if not self._filters:
+            return ''
+        exprs = (e.build(FilterExpression) for e in self._filters)
+        result = FilterExpression.operator[Operator.OR].join(exprs)
+        return '--filter %s' % utils.escape(result, True)
+
     def _condition(self):
         return ' '.join((
             self._make_match_columns(),
             QueryOptionsMixin._condition(self),
             self._makecache(),
             self._makematch_escalation_threshold(),
-            self._makeparams())).strip()
+            self._makeparams(),
+            self._makefilters())).strip()
 
     def __str__(self):
-        return ('select --table "%s" %s' % (
+        return utils.to_text('select --table %s %s' % (
             self._table.__tablename__,
             self._condition())).strip()
 
@@ -398,16 +420,16 @@ class SelectQuery(SelectQueryBase):
         return DrillDownQuery(self, *columns)
 
     def _makeparams(self):
-        params = [utils.escape('%s:@"%s"' % target) for target in
+        params = ['%s:@%s' % (k, utils.escape(v, True)) for k, v in
                   sorted(self._target.items())]
-        param = Expression.OR.join(params)
-        expr = Expression.OR.join(utils.escape(str(expr)) for expr in
-                                  self._exprs)
+        op = QueryExpression.operator[Operator.BIT_OR]
+        param = op.join(params)
+        expr = op.join(expr.build(QueryExpression) for expr in self._exprs)
         result = param and '(%s)' % param
         if result and expr:
-            result += Expression.AND
+            result += QueryExpression.operator[Operator.BIT_AND]
         result += expr
-        return '--query "%s"' % result if result else ''
+        return '--query %s' % utils.escape(result, True) if result else ''
 
 
 class DrillDownQuery(SelectQueryBase, QueryOptionsMixin):
@@ -444,112 +466,286 @@ class DrillDownQuery(SelectQueryBase, QueryOptionsMixin):
         return str(self.parent) + (' %s' % self._condition())
 
 
-class MatchColumn(object):
-    """match column representation class
-    """
-    INITIAL_WEIGHT = 1
-
-    def __init__(self, column, weight=INITIAL_WEIGHT):
-        self.column = column
-        self.weight = weight
-
-    def __mul__(self, other):
-        return MatchColumn(self.column, int(other))
-
-    def __or__(self, other):
-        return MatchColumnsTree(self, other)
-
-    def __str__(self):
-        result = self.column.name
-        if self.weight > MatchColumn.INITIAL_WEIGHT:
-            result += ' * %s' % self.weight
-        return result
-
-
-class MatchColumnsTree(object):
-    """Match columns tree"""
-
-    __slots__ = ['left', 'right']
-
-    def __init__(self, left, right):
-        self.left = left
-        self.right = right
-
-    def __or__(self, other):
-        return MatchColumnsTree(self, other)
-
-    def _extract_tree(self, inst):
-        if isinstance(inst, MatchColumnsTree):
-            return '%s || %s' % (
-                self._extract_tree(inst.left),
-                self._extract_tree(inst.right))
-        elif isinstance(inst, MatchColumn):
-            return str(inst)
-        else:
-            raise ValueError(
-                "instance takes only the `MatchColumn` and `MatchColumnsTree`."
-                " but `%s` given." % repr(inst))
-
-    def __str__(self):
-        return self._extract_tree(self)
+class Operator(object):
+    EQUAL = 'EQUAL'
+    GREATER_EQUAL = 'GREATER_EQUAL'
+    GREATER_THAN = 'GREATER_THAN'
+    LESS_EQUAL = 'LESS_EQUAL'
+    LESS_THAN = 'LESS_THAN'
+    NOT_EQUAL = 'NOT_EQUAL'
+    OR = 'OR'
+    AND = 'AND'
+    NOT = 'NOT'
+    DIFF = 'DIFF'
+    INVERT = 'INVERT'
+    BIT_AND = 'BIT_AND'
+    BIT_OR = 'BIT_OR'
+    BIT_XOR = 'BIT_XOR'
+    LSHIFT = 'LSHIFT'
+    RSHIFT = 'RSHIFT'
+    ADD = 'ADD'
+    SUB = 'SUB'
+    MUL = 'MUL'
+    DIV = 'DIV'
+    MOD = 'MOD'
+    IADD = 'IADD'
+    ISUB = 'ISUB'
+    IMUL = 'IMUL'
+    IDIV = 'IDIV'
+    IMOD = 'IMOD'
+    ILSHIFT = 'ILSHIFT'
+    IRSHIFT = 'IRSHIFT'
+    IBIT_AND = 'IBIT_AND'
+    IBIT_OR = 'IBIT_OR'
+    IBIT_XOR = 'IBIT_XOR'
+    MATCH = 'MATCH'
+    STARTSWITH = 'STARTSWITH'
+    ENDSWITH = 'ENDSWITH'
+    NEAR = 'NEAR'
+    SIMILAR = 'SIMILAR'
+    TERM_EXTRACT = 'TERM_EXTRACT'
 
 
 class BaseExpression(object):
+    __slots__ = []
+
+    @property
+    def lvalue(self):
+        return self
+
     def __eq__(self, other):
-        return ExpressionTree(Expression.EQUAL, self, other)
+        return ExpressionTree(Operator.EQUAL, self.lvalue, other)
 
     def __ge__(self, other):
-        return ExpressionTree(Expression.GREATER_EQUAL, self, other)
+        return ExpressionTree(Operator.GREATER_EQUAL, self.lvalue, other)
 
     def __gt__(self, other):
-        return ExpressionTree(Expression.GREATER_THAN, self, other)
+        return ExpressionTree(Operator.GREATER_THAN, self.lvalue, other)
 
     def __le__(self, other):
-        return ExpressionTree(Expression.LESS_EQUAL, self, other)
+        return ExpressionTree(Operator.LESS_EQUAL, self.lvalue, other)
 
     def __lt__(self, other):
-        return ExpressionTree(Expression.LESS_THAN, self, other)
+        return ExpressionTree(Operator.LESS_THAN, self.lvalue, other)
 
     def __ne__(self, other):
-        return ExpressionTree(Expression.NOT_EQUAL, self, other)
+        return ExpressionTree(Operator.NOT_EQUAL, self.lvalue, other)
 
-    def __and__(self, other):
-        return ExpressionTree(Expression.AND, self, other)
-
-    def __or__(self, other):
-        return ExpressionTree(Expression.OR, self, other)
+    def __add__(self, other):
+        return ExpressionTree(Operator.ADD, self.lvalue, other)
 
     def __sub__(self, other):
-        return ExpressionTree(Expression.NOT, self, other)
+        return ExpressionTree(Operator.SUB, self.lvalue, other)
+
+    def __mul__(self, other):
+        return ExpressionTree(Operator.MUL, self.lvalue, other)
+
+    def __truediv__(self, other):
+        return ExpressionTree(Operator.DIV, self.lvalue, other)
+
+    def __div__(self, other):
+        return self.__truediv__(other)
+
+    def __mod__(self, other):
+        return ExpressionTree(Operator.MOD, self.lvalue, other)
+
+    def not_(self):
+        return ExpressionTree(Operator.NOT, None, self.lvalue)
+
+    def and_(self, other):
+        return ExpressionTree(Operator.AND, self.lvalue, other)
+
+    def or_(self, other):
+        return ExpressionTree(Operator.OR, self.lvalue, other)
+
+    def diff(self, other):
+        return ExpressionTree(Operator.DIFF, self.lvalue, other)
+
+    def __invert__(self):
+        return ExpressionTree(Operator.INVERT, None, self.lvalue)
+
+    def __and__(self, other):
+        return ExpressionTree(Operator.BIT_AND, self.lvalue, other)
+
+    def __or__(self, other):
+        return ExpressionTree(Operator.BIT_OR, self.lvalue, other)
+
+    def __xor__(self, other):
+        return ExpressionTree(Operator.BIT_XOR, self.lvalue, other)
+
+    def __lshift__(self, other):
+        return ExpressionTree(Operator.LSHIFT, self.lvalue, other)
+
+    def __rshift__(self, other):
+        return ExpressionTree(Operator.RSHIFT, self.lvalue, other)
+
+    def __iadd__(self, other):
+        return ExpressionTree(Operator.IADD, self.lvalue, other)
+
+    def __isub__(self, other):
+        return ExpressionTree(Operator.ISUB, self.lvalue, other)
+
+    def __imul__(self, other):
+        return ExpressionTree(Operator.IMUL, self.lvalue, other)
+
+    def __itruediv__(self, other):
+        return ExpressionTree(Operator.IDIV, self.lvalue, other)
+
+    def __idiv__(self, other):
+        return self.__itruediv__(other)
+
+    def __imod__(self, other):
+        return ExpressionTree(Operator.IMOD, self.lvalue, other)
+
+    def __ilshift__(self, other):
+        return ExpressionTree(Operator.ILSHIFT, self.lvalue, other)
+
+    def __irshift__(self, other):
+        return ExpressionTree(Operator.IRSHIFT, self.lvalue, other)
+
+    def __iand__(self, other):
+        return ExpressionTree(Operator.IBIT_AND, self.lvalue, other)
+
+    def __ior__(self, other):
+        return ExpressionTree(Operator.IBIT_OR, self.lvalue, other)
+
+    def __ixor__(self, other):
+        return ExpressionTree(Operator.IBIT_XOR, self.lvalue, other)
+
+    def match(self, other):
+        return ExpressionTree(Operator.MATCH, self.lvalue, other)
+
+    def startswith(self, other):
+        return ExpressionTree(Operator.STARTSWITH, self.lvalue, other)
+
+    def endswith(self, other):
+        return ExpressionTree(Operator.ENDSWITH, self.lvalue, other)
+
+    def near(self, other):
+        return ExpressionTree(Operator.NEAR, self.lvalue, other)
+
+    def similar(self, other):
+        return ExpressionTree(Operator.SIMILAR, self.lvalue, other)
+
+    def term_extract(self, other):
+        return ExpressionTree(Operator.TERM_EXTRACT, self.lvalue, other)
 
 
+@utils.python_2_unicode_compatible
 class Expression(BaseExpression):
     """Expression constants"""
 
     __slots__ = ['value']
 
-    EQUAL = ':'
-    GREATER_EQUAL = ':>='
-    GREATER_THAN = ':>'
-    LESS_EQUAL = ':<='
-    LESS_THAN = ':<'
-    NOT_EQUAL = ':!'
-    OR = ' OR '
-    AND = ' + '
-    NOT = ' - '
-
     def __init__(self, value):
+        super(Expression, self).__init__()
         self.value = value
 
     @classmethod
     def wrap_expr(cls, *args):
-        return (Expression(arg) if
-                not isinstance(arg, (Expression, ExpressionTree))
-                else arg for arg in args)
+        return (cls._wrap(e) for e in args)
+
+    @classmethod
+    def _wrap(cls, expr):
+        if expr is None or isinstance(expr, (Expression, ExpressionTree)):
+            return expr
+        return Expression(expr)
+
+    def build(self, expr_cls):
+        return str(expr_cls(self))
 
     def __str__(self):
-        expr_str = str(self.value)
-        return '"%s"' % expr_str if ' ' in expr_str else expr_str
+        return utils.to_text(self.value)
+
+
+class MatchColumn(Expression):
+    """Match column class"""
+
+    operator = {
+        Operator.OR: ' || ',
+        Operator.MUL: ' * ',
+        }
+
+
+@utils.python_2_unicode_compatible
+class QueryExpression(Expression):
+    """Query expression class"""
+
+    operator = {
+        Operator.EQUAL: ':',
+        Operator.GREATER_EQUAL: ':>=',
+        Operator.GREATER_THAN: ':>',
+        Operator.LESS_EQUAL: ':<=',
+        Operator.LESS_THAN: ':<',
+        Operator.NOT_EQUAL: ':!',
+        Operator.BIT_OR: ' OR ',
+        Operator.BIT_AND: ' + ',
+        Operator.SUB: ' - ',
+        }
+
+    def __str__(self):
+        expr_str = utils.to_text(self.value)
+        return utils.escape(expr_str)
+
+
+@utils.python_2_unicode_compatible
+class FilterExpression(Expression):
+    """Filter expression class"""
+
+    operator = {
+        Operator.ADD: ' + ',
+        Operator.SUB: ' - ',
+        Operator.MUL: ' * ',
+        Operator.DIV: ' / ',
+        Operator.MOD: ' % ',
+        Operator.NOT: '!',
+        Operator.AND: ' && ',
+        Operator.OR: ' || ',
+        Operator.DIFF: ' &! ',
+        Operator.INVERT: '~',
+        Operator.BIT_AND: ' & ',
+        Operator.BIT_OR: ' | ',
+        Operator.BIT_XOR: ' ^ ',
+        Operator.LSHIFT: ' << ',
+        Operator.RSHIFT: ' >>> ',
+        Operator.EQUAL: ' == ',
+        Operator.NOT_EQUAL: ' != ',
+        Operator.LESS_THAN: ' < ',
+        Operator.LESS_EQUAL: ' <= ',
+        Operator.GREATER_THAN: ' > ',
+        Operator.GREATER_EQUAL: ' >= ',
+        Operator.IADD: ' += ',
+        Operator.ISUB: ' -= ',
+        Operator.IMUL: ' *= ',
+        Operator.IDIV: ' /= ',
+        Operator.IMOD: ' %= ',
+        Operator.ILSHIFT: ' <<= ',
+        Operator.IRSHIFT: ' >>>= ',
+        Operator.IBIT_AND: ' &= ',
+        Operator.IBIT_OR: ' |= ',
+        Operator.IBIT_XOR: ' ^= ',
+        Operator.MATCH: ' @ ',
+        Operator.STARTSWITH: ' @^ ',
+        Operator.ENDSWITH: ' @$ ',
+        Operator.NEAR: ' *N ',
+        Operator.SIMILAR: ' *S ',
+        Operator.TERM_EXTRACT: ' *T ',
+        }
+
+    def __str__(self):
+        value = self.value
+        if value is True:
+            expr_str = 'true'
+        elif value is False:
+            expr_str = 'false'
+        elif value is None:
+            expr_str = 'null'
+        elif isinstance(value, (datetime, date)):
+            expr_str = utils.escape(value.strftime('%Y/%m/%d %H:%M:%S.%f'))
+        else:
+            expr_str = utils.escape(utils.to_text(value))
+        # TODO: Geo point, Array
+        return expr_str
 
 GE = Expression
 
@@ -557,23 +753,30 @@ GE = Expression
 class ExpressionTree(BaseExpression):
     """Query conditional expression tree class"""
 
-    __slots__ = ['expr', 'left', 'right']
+    __slots__ = ['op', 'left', 'right']
 
-    def __init__(self, expr, left, right):
-        self.expr = expr
+    def __init__(self, op, left, right):
+        super(ExpressionTree, self).__init__()
+        self.op = op
         self.left, self.right = tuple(Expression.wrap_expr(left, right))
 
-    def __str__(self):
-        return self._extract_tree(self)
+    def build(self, expr_cls):
+        return self._extract_tree(self, expr_cls)
 
-    def _extract_tree(self, expr):
-        if isinstance(expr, ExpressionTree):
-            return '(%s%s%s)' % (
-                self._extract_tree(expr.left), expr.expr,
-                self._extract_tree(expr.right),
-                )
-        else:
-            return str(expr)
+    def _extract_tree(self, expr, expr_cls):
+        if expr is None:
+            return ''
+        if not isinstance(expr, ExpressionTree):
+            return expr_cls(expr)
+        op = expr_cls.operator.get(expr.op, None)
+        if op is None:
+            raise NotImplementedError(
+                "An operator `%s` is not defined in `%s`" %
+                (expr.op, expr_cls.__name__))
+        return '(%s%s%s)' % (
+            self._extract_tree(expr.left, expr_cls), op,
+            self._extract_tree(expr.right, expr_cls),
+            )
 
 
 class LoadQuery(Query):
@@ -617,15 +820,14 @@ class LoadQuery(Query):
         self._data = None
 
     def _makejson(self):
-        return utils.escape(json.dumps([v.asdict(excludes=('_id',)) for v in
-                                        self._data]))
+        return json.dumps([v.asdict(excludes=('_id',)) for v in self._data])
 
     def __str__(self):
         return ' '.join((
             'load',
             '--table', self._table.__tablename__,
             '--input-type', 'json',
-            '--values', '"%s"' % self._makejson()))
+            '--values', utils.escape(self._makejson(), True)))
 
 
 class SimpleQuery(Query):
@@ -655,7 +857,7 @@ class SimpleQuery(Query):
         if id is not None:
             query.extend(('--id', str(id)))
         if filter is not None:
-            query.extend(('--filter', '"%s"' % utils.escape(filter)))
+            query.extend(('--filter', utils.escape(str(filter), True)))
         self._query.extend(query)
         return self
 
@@ -870,7 +1072,7 @@ class SuggestQuery(Query, QueryOptionsMixin):
             '--column', '"%s"' % self._table.kana.name,
             '--types', '"%s"' % self._types,
             self._condition(),
-            '--query', '"%s"' % utils.escape(self._query)))
+            '--query', utils.escape(self._query, True)))
 
 
 class SuggestLoadQuery(LoadQuery):
@@ -883,4 +1085,4 @@ class SuggestLoadQuery(LoadQuery):
             '--input_type', 'json',
             '--each',
             "'suggest_preparer(_id, type, item, sequence, time, pair_query)'",
-            '"%s"' % self._makejson()))
+            utils.escape(self._makejson(), True)))
