@@ -3,7 +3,6 @@
 # Copyright (c) 2013 Naoya Inada <naoina@kuune.org>
 # Licensed under the MIT License.
 
-
 __author__ = "Naoya Inada <naoina@kuune.org>"
 
 __all__ = [
@@ -25,12 +24,10 @@ from pyroonga.odm.attributes import (
     Tokenizer,
     NormalizerSymbol
     )
+from pyroonga.odm import query
 from pyroonga.odm.query import (
-    Expression,
-    ExpressionTree,
     GroongaRecord,
     LoadQuery,
-    MatchColumn,
     SuggestQuery,
     SuggestLoadQuery,
     SelectQuery,
@@ -300,7 +297,7 @@ class TableBase(object):
         return result
 
 
-class Column(object):
+class Column(query.BaseExpression):
     """Column representation class
 
     e.g. ::
@@ -342,34 +339,14 @@ class Column(object):
         self.tablename = self.name = self.value = None
         self._desc = False
 
-    def __eq__(self, other):
-        return ExpressionTree(Expression.EQUAL, self.name, other)
-
-    def __ge__(self, other):
-        return ExpressionTree(Expression.GREATER_EQUAL, self.name, other)
-
-    def __gt__(self, other):
-        return ExpressionTree(Expression.GREATER_THAN, self.name, other)
-
-    def __le__(self, other):
-        return ExpressionTree(Expression.LESS_EQUAL, self.name, other)
-
-    def __lt__(self, other):
-        return ExpressionTree(Expression.LESS_THAN, self.name, other)
-
-    def __ne__(self, other):
-        return ExpressionTree(Expression.NOT_EQUAL, self.name, other)
+    @property
+    def lvalue(self):
+        return self.name
 
     def __neg__(self):
         self._desc = True
         return self
 
-    def __mul__(self, other):
-        return MatchColumn(self).__mul__(other)
-
-    def __or__(self, other):
-        return MatchColumn(self).__or__(other)
-
     def __str__(self):
         if not (self.tablename and self.name):
             raise TypeError('column instance is not initialized')
