@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 import json
-import operator
 import random
 
 import pytest
@@ -18,7 +17,6 @@ from pyroonga.odm.attributes import (
     )
 from pyroonga.odm.table import (
     Column,
-    TableMeta,
     SuggestTable,
     prop_attr,
     tablebase,
@@ -30,126 +28,11 @@ from pyroonga.odm.query import (
     GroongaSuggestResults,
     GroongaSuggestResult,
     LoadQuery,
-    MatchColumn,
-    MatchColumnsTree,
     )
 from pyroonga import utils
 from pyroonga.tests import utils as test_utils
 
 
-class TestColumn(object):
-    def test___init___with_default_value(self):
-        col = Column()
-        assert col.flags is ColumnFlags.COLUMN_SCALAR
-        assert col.type is DataType.ShortText
-        assert col.__tablemeta__ is TableMeta
-        assert col.tablename is None
-        assert col.name is None
-        assert col.value is None
-
-    def test___init___with_invalid_flags(self):
-        with pytest.raises(TypeError):
-            Column(flags='')
-
-    def test___init___with_flags(self):
-        try:
-            col = Column(flags=ColumnFlags.COLUMN_VECTOR)
-            assert col.flags is ColumnFlags.COLUMN_VECTOR
-        except TypeError:
-            assert False, "TypeError should not be raised"
-
-    @pytest.mark.parametrize('type_', (
-        DataType.UInt32,
-        TableMeta,
-    ))
-    def test___init___with_type(self, type_):
-        try:
-            col = Column(type=type_)
-            assert col.type is type_
-        except TypeError:
-            assert False, "TypeError should not be raised"
-
-    def test___mul__(self):
-        col = Column()
-        expected = random.randrange(1000)
-        result = col.__mul__(expected)
-        assert isinstance(result, MatchColumn)
-        assert result.column is col
-        assert result.weight == expected
-
-    def test___mul___with_implicit_calls(self):
-        col = Column()
-        expected = random.randrange(1000)
-        result = col * expected
-        assert isinstance(result, MatchColumn)
-        assert result.column is col
-        assert result.weight == expected
-
-    def test___or__(self):
-        col1, col2 = Column(), Column()
-        result = col1.__or__(col2)
-        assert isinstance(result, MatchColumnsTree)
-        assert result.left.column is col1
-        assert result.right is col2
-
-    def test___or___with_implicit_calls(self):
-        col1, col2 = Column(), Column()
-        result = col1 | col2
-        assert isinstance(result, MatchColumnsTree)
-        assert result.left.column is col1
-        assert result.right is col2
-
-    def test___str___without_attributes(self):
-        col = Column(flags=ColumnFlags.COLUMN_SCALAR, type=DataType.ShortText)
-        with pytest.raises(TypeError):
-            str(col)
-
-    def test___str___without_name(self):
-        col = Column(flags=ColumnFlags.COLUMN_SCALAR, type=DataType.ShortText)
-        col.tablename = 'tb1'
-        with pytest.raises(TypeError):
-            str(col)
-
-    def test___str___without_tablename(self):
-        col = Column(flags=ColumnFlags.COLUMN_SCALAR, type=DataType.ShortText)
-        col.name = 'name1'
-        with pytest.raises(TypeError):
-            str(col)
-
-    def test___str___without_table(self):
-        col = Column(flags=ColumnFlags.COLUMN_VECTOR, type=DataType.ShortText)
-        col.tablename = 'tb2'
-        col.name = 'name2'
-        assert str(col) == ('column_create --table tb2 --name name2 --flags'
-                            ' COLUMN_VECTOR --type ShortText')
-
-    def test___str___with_class(self):
-        ExampleTableBase = tablebase()
-
-        class ExampleTable(ExampleTableBase):
-            name = Column()
-
-        col = Column(flags=ColumnFlags.COLUMN_INDEX, type=ExampleTable,
-                     source=ExampleTable.name)
-        col.tablename = 'tb3'
-        col.name = 'name3'
-        assert str(col) == ('column_create --table tb3 --name name3 --flags'
-                            ' COLUMN_INDEX --type ExampleTable --source name')
-
-    def test___str___with_class_string(self):
-        ExampleTableBase = tablebase()
-
-        class ExampleTable(ExampleTableBase):
-            name = Column()
-
-        col = Column(flags=ColumnFlags.COLUMN_INDEX, type='ExampleTable',
-                     source='name')
-        col.tablename = 'tb4'
-        col.name = 'name4'
-        assert str(col) == ('column_create --table tb4 --name name4 --flags'
-                            ' COLUMN_INDEX --type ExampleTable --source name')
-
-
 class TestPropAttr(object):
     def test_prop_attr(self):
         class TestClass(object):
@@ -493,11 +376,18 @@ class TestTable(object):
         Table.bind(grn)
         test_utils.sendquery('table_create --name %s --flags TABLE_HASH_KEY '
                         '--key_type ShortText' % Tb.__tablename__)
-        self._insert(Tb.__tablename__, [{'_key': 'key1'}, {'_key': 'key2'}])
+        self._insert(Tb.__tablename__, [
+            {'_key': 'key1'},
+            {'_key': 'key2'},
+            {'_key': u'キー３'},
+            {'_key': 'キー４'},
+            ])
         result = Tb.select().all()
         expected = [{'_id': 1, '_key': 'key1'},
-                    {'_id': 2, '_key': 'key2'}]
-        self.assertGroongaResultEqual(result, expected, all_len=2)
+                    {'_id': 2, '_key': 'key2'},
+                    {'_id': 3, '_key': u'キー３'},
+                    {'_id': 4, '_key': u'キー４'}]
+        self.assertGroongaResultEqual(result, expected, all_len=4)
 
     def test_select_with_column(self, Table):
         class Tb(Table):
@@ -516,6 +406,8 @@ class TestTable(object):
             {'_key': 'key1', 'name': 'Name1', 'address': 'Address1'},
             {'_key': 'key2', 'name': 'name2', 'address': 'address2'},
             {'_key': 'key3', 'name': 'foo', 'address': 'bar'},
+            {'_key': u'キー４', 'name': '園城寺怜', 'address': u'千里山'},
+            {'_key': 'キー５', 'name': u'白水哩', 'address': '新道寺'},
             ])
 
         result = Tb.select(_key='1').all()
@@ -528,18 +420,39 @@ class TestTable(object):
                      'name': 'name2'}]
         self.assertGroongaResultEqual(result, expected, all_len=1)
 
+        result = Tb.select(_key='キー４').all()
+        expected = [{'_id': 4, '_key': u'キー４', 'address': u'千里山',
+                     'name': u'園城寺怜'}]
+        self.assertGroongaResultEqual(result, expected, all_len=1)
+
+        result = Tb.select(_key=u'キー５').all()
+        expected = [{'_id': 5, '_key': u'キー５', 'address': u'新道寺',
+                     'name': u'白水哩'}]
+        self.assertGroongaResultEqual(result, expected, all_len=1)
+
         result = Tb.select(name='name').all()
         expected = [{'_id': 1, '_key': 'key1', 'address': 'Address1',
                      'name': 'Name1'},
                     {'_id': 2, '_key': 'key2', 'address': 'address2',
-                     'name': 'name2'}]
+                     'name': 'name2'},
+                    ]
         self.assertGroongaResultEqual(result, expected, all_len=2)
 
+        result = Tb.select(name='城').all()
+        expected = [{'_id': 4, '_key': u'キー４', 'address': u'千里山',
+                     'name': u'園城寺怜'}]
+        self.assertGroongaResultEqual(result, expected, all_len=1)
+
         result = Tb.select(address='ar').all()
         expected = [{'_id': 3, '_key': 'key3', 'address': 'bar',
                      'name': 'foo'}]
         self.assertGroongaResultEqual(result, expected, all_len=1)
 
+        result = Tb.select(address='道寺').all()
+        expected = [{'_id': 5, '_key': u'キー５', 'address': u'新道寺',
+                     'name': u'白水哩'}]
+        self.assertGroongaResultEqual(result, expected, all_len=1)
+
     def test_select_with_condition(self, Table1, fixture1):
         Tb, fixture = Table1, fixture1
 
@@ -861,32 +774,40 @@ class TestTable(object):
 
         data = [Tb(_key='key1', name='name1'),
                 Tb(_key='key2', name='name2'),
-                Tb(_key='key3', name='name3')]
+                Tb(_key='key3', name='name3'),
+                Tb(_key='キー４', name='園城寺怜'),
+                Tb(_key=u'キー５', name=u'白水哩')]
         result = Tb.load(data)
-        assert (result == 3) is True
+        assert (result == 5) is True
         stored = json.loads(test_utils.sendquery(
             'select --table %s' % Tb.__tablename__))
-        expected = [[[3],
+        expected = [[[5],
                      [['_id', 'UInt32'], ['_key', 'ShortText'],
                       ['name', 'ShortText']],
                      [1, 'key1', 'name1'],
                      [2, 'key2', 'name2'],
-                     [3, 'key3', 'name3']]]
+                     [3, 'key3', 'name3'],
+                     [4, u'キー４', u'園城寺怜'],
+                     [5, u'キー５', u'白水哩']]]
         assert stored[1] == expected
 
         # data override test
         data = [Tb(_key='key1', name='foo'),
                 Tb(_key='key2', name='bar'),
-                Tb(_key='key3', name='baz')]
+                Tb(_key='key3', name='baz'),
+                Tb(_key='キー４', name='松実玄'),
+                Tb(_key=u'キー５', name='新子憧')]
         result = Tb.load(data)
         stored = json.loads(test_utils.sendquery(
             'select --table %s' % Tb.__tablename__))
-        expected = [[[3],
+        expected = [[[5],
                      [['_id', 'UInt32'], ['_key', 'ShortText'],
                       ['name', 'ShortText']],
                      [1, 'key1', 'foo'],
                      [2, 'key2', 'bar'],
-                     [3, 'key3', 'baz']]]
+                     [3, 'key3', 'baz'],
+                     [4, u'キー４', u'松実玄'],
+                     [5, u'キー５', u'新子憧']]]
         assert stored[1] == expected
 
     def test_load_lazy(self, Table):
@@ -903,7 +824,9 @@ class TestTable(object):
 
         data = [Tb(_key='key1', name='name1'),
                 Tb(_key='key2', name='name2'),
-                Tb(_key='key3', name='name3')]
+                Tb(_key='key3', name='name3'),
+                Tb(_key='キー４', name='園城寺怜'),
+                Tb(_key=u'キー５', name=u'白水哩')]
         result = Tb.load(data, immediate=False)
         assert isinstance(result, LoadQuery)
         stored = json.loads(test_utils.sendquery(
@@ -915,12 +838,14 @@ class TestTable(object):
         result.commit()  # load actually
         stored = json.loads(test_utils.sendquery(
             'select --table %s' % Tb.__tablename__))
-        expected = [[[3],
+        expected = [[[5],
                      [['_id', 'UInt32'], ['_key', 'ShortText'],
                       ['name', 'ShortText']],
                      [1, 'key1', 'name1'],
                      [2, 'key2', 'name2'],
-                     [3, 'key3', 'name3']]]
+                     [3, 'key3', 'name3'],
+                     [4, u'キー４', u'園城寺怜'],
+                     [5, u'キー５', u'白水哩']]]
         assert stored[1] == expected
 
         # duplicate commit
@@ -953,7 +878,8 @@ class TestTable(object):
         data2 = [Tb(_key='key4', name='Madoka Kaname'),
                  Tb(_key='key5', name='Homura Akemi'),
                  Tb(_key='key6', name='Kyoko Sakura'),
-                 Tb(_key='key7', name='Sayaka Miki')]
+                 Tb(_key='key7', name='Sayaka Miki'),
+                 Tb(_key=u'キー８', name=u'巴マミ')]
         result.load(data2)
         stored = json.loads(test_utils.sendquery(
             'select --table %s' % Tb.__tablename__))
@@ -964,7 +890,7 @@ class TestTable(object):
         result.commit()  # load actually
         stored = json.loads(test_utils.sendquery(
             'select --table %s' % Tb.__tablename__))
-        expected = [[[7],
+        expected = [[[8],
                      [['_id', 'UInt32'], ['_key', 'ShortText'],
                       ['name', 'ShortText']],
                      [1, 'key1', 'name1'],
@@ -973,7 +899,8 @@ class TestTable(object):
                      [4, 'key4', 'Madoka Kaname'],
                      [5, 'key5', 'Homura Akemi'],
                      [6, 'key6', 'Kyoko Sakura'],
-                     [7, 'key7', 'Sayaka Miki']]]
+                     [7, 'key7', 'Sayaka Miki'],
+                     [8, u'キー８', u'巴マミ']]]
         assert stored[1] == expected
 
     def test_load_lazy_rollback(self, Table):
@@ -1432,23 +1359,25 @@ class TestTable(object):
         class A(Table):
             foo = None
             bar = None
-        expected1, expected2 = (test_utils.random_string(),
-                                test_utils.random_string())
-        table = A(foo=expected1, bar=expected2)
+            baz = None
+        expected1 = test_utils.random_string()
+        expected2 = '園城寺怜'
+        expected3 = u'白水哩'
+        table = A(foo=expected1, bar=expected2, baz=expected3)
         result = table.asdict()
-        assert result == {'foo': expected1, 'bar': expected2}
+        assert result == {'foo': expected1, 'bar': expected2, 'baz': expected3}
 
     @pytest.mark.parametrize(('excludes', 'expected'), (
-        (['foo'], {'bar': 'baz', 'baz': 'hoge'}),
-        (['bar'], {'foo': 'bar', 'baz': 'hoge'}),
-        (['foo', 'baz'], {'bar': 'baz'}),
+        (['foo'], {'bar': '園城寺怜', 'baz': u'白水哩'}),
+        (['bar'], {'foo': 'bar', 'baz': u'白水哩'}),
+        (['foo', 'baz'], {'bar': '園城寺怜'}),
     ))
     def test_asdict_with_excludes(self, Table, excludes, expected):
         class A(Table):
             foo = None
             bar = None
             baz = None
-        table = A(foo='bar', bar='baz', baz='hoge')
+        table = A(foo='bar', bar='園城寺怜', baz=u'白水哩')
         result = table.asdict(excludes=excludes)
         assert result == expected
 
